#include"com/mesonbuild/simple.pb.h"
#include"com/mesonbuild/subsite/complex.pb.h"

#include<memory>

int main(int argc, char **argv) {
    GOOGLE_PROTOBUF_VERIFY_VERSION;
    {
        subdirectorial::SimpleMessage *s = new subdirectorial::SimpleMessage();
        s->set_the_integer(3);
        subdirectorial::ComplexMessage c;
        c.set_allocated_sm(s);
    }
    google::protobuf::ShutdownProtobufLibrary();
    return 0;
}
