#include <zlib.h>
#include <math.h>

#ifndef DEFINED
#error expected compile_arg not found
#endif

double zero;
int ok(void) {
    void * something = deflate;
    if(something != 0)
        return 0;
    return (int)cos(zero);
}
