---
short-description: Meson functions
render-subpages: false
...

# Functions

This document lists all functions available in `meson.build` files.
See the [root manual document]({{root}}) for
an overview of all features.

{{#functions}}
<p style="padding: 7.5px; margin: 0px;"></p>

{{>func}}

<p style="padding: 7.5px; margin: 0px;"></p>

---
{{/functions}}
