<div style="margin: 0px; text-align: center;">
{{#since}}
<p style="margin: 0px;"><em style="color: #5affff;">(since {{since}})</em></p>
{{/since}}
{{#deprecated}}
<div style="color: #ffa844">
    <p style="margin: 0px;"><strong>DEPRECATED</strong></p>
    <p style="margin: 0px;"><em>in {{deprecated}}</em></p>
</div>
{{/deprecated}}
{{#optional}}
<p style="margin: 0px;"><b>[optional]</b></p>
{{/optional}}
{{#default}}
<p style="margin: 0px;"><code class="language-meson">default =
{{default}}</code></p>
{{/default}}
</div>
